@@ -71,9 +71,9 @@ end
 
 
 class Draw_data
-  attr_accessor :filename, :filetype
-  attr_accessor :gphys
+  attr_accessor :gphys, :gphys2
   attr_accessor :cont, :tone
+  attr_accessor :vector
   attr_accessor :transpose
   attr_accessor :type, :map_type
   attr_accessor :title
@@ -86,6 +86,7 @@ class Draw_data
   end
   def clear_var
     @gphys = nil
+    @gphys2 = nil
     @transpose = false
     @axis_x.clear
     @axis_y.clear
@@ -131,26 +132,26 @@ class Mouse_Events
       if @axis==4 then
 	@move=MOVE
 	@bpx,@bpy, = px,py
-	p "begin move"
+	p "begin move" if $DEBUG
       else
 	@move=RESIZE
-	p "begin resize"
+ 	p "begin resize" if $DEBUG
       end
     else
       @move = false
     end
   end
   def released(px,py)
-    print "released:",px,",",py,"\n"
+    print "released:",px,",",py,"\n" if $DEBUG
     if @move==MOVE then
       @move=false
-      p "end move"
+      p "end move" if $DEBUG
       dx=px-@bpx
       dy=py-@bpy
       [@left+dx,@right+dx,@bottom+dy,@top+dy]
     elsif @move==RESIZE then
       @move=false
-      p "end resize"
+      p "end resize" if $DEBUG
       l,r,b,t, = @left,@right,@bottom,@top
       @left=nil;@right=nil;@bottom=nil;@top=nil
       @bpx=nil;@bpy=nil
@@ -233,7 +234,8 @@ end
 
 
 class Draw
-  def initialize
+  def initialize(n)
+    @winno = n
     @pixmap = nil
     @height=400
     @width=400
@@ -252,14 +254,16 @@ class Draw
     @frame = Mouse_Events.new(@height,@width)
     @opened = false
   end
+  def show
+    @window.show if defined?(@window)&&(!@window.nil?)
+  end
   def draw_x(draw_data)
-    @filename = draw_data.filename
-    @filetype = draw_data.filetype
     @xmin = draw_data.axis_x.min
     @xmax = draw_data.axis_x.max
     @ymin = draw_data.axis_y.min
     @ymax = draw_data.axis_y.max
     @gphys = draw_data.gphys
+    @gphys2 = draw_data.gphys2
     @mapping = draw_data.gphys.data.mapping.slicer
 #    @miss = draw_data.missing_value
     @title = draw_data.title
@@ -274,7 +278,9 @@ class Draw
     @maptype = draw_data.map_type
     @cont = draw_data.cont
     @tone = draw_data.tone
-    @fig.var_set(@filename,@filetype,@gphys.funcs,@mapping)
+    @vector = draw_data.vector
+    @fig.var_set(@gphys.funcs,@mapping,0)
+    @fig.var_set(@gphys2.funcs,@mapping,1) if @type == VECTOR
     @fig.title_set(@x_title,@x_unit,@y_title,@y_unit, __FILE__,__LINE__)
     if @window.nil?
       open_graphics
@@ -297,7 +303,7 @@ class Draw
       if p.button==1 then
 	px = (p.x-@frame.xoffset)/@frame.length
 	py = 1 - (p.y-@frame.yoffset)/@frame.length
-	print "press:",px,",",py,"\n"
+	print "press:",px,",",py,"\n" if $DEBUG
 	@frame.pressed(px,py,@vxmin,@vxmax,@vymin,@vymax)
       else
 	w.get_toplevel.hide
@@ -326,16 +332,17 @@ class Draw
     @window.signal_connect("delete-event"){|w,p| w.hide; true}
     @window.signal_connect("destroy"){exit}
     @window.signal_connect("key-press-event"){|w,e|
-      key = e.string
-      p key+" key press"
+      key = Gdk::Keyval.to_name(e.keyval)
+      p key+" key press" if $DEBUG
       if key=="q" then
 	w.hide
-      elsif key=="\r" || key==" " then
+      elsif key=="space" || key=="Return" then
 	$main.draw_next
       end
     }
     @window.add(aspect_frame.show_all)
-    @window.set_title(@filename[0])
+    names = @gphys.funcs[0]
+    @window.set_title("#{@winno}: #{names[2]}@#{names[0]}")
     @window.show
   end
   def save_code(filename)
@@ -367,6 +374,7 @@ class Draw
     DCL::swlset("LWAIT", false )
     @fig.parm_set('DCL::sglset("LCNTL", false )', __FILE__,__LINE__)
     @fig.parm_set('DCL::udlset("LMSG", false )', __FILE__,__LINE__)
+    @fig.parm_set('DCL::uglset("LMSG", false )', __FILE__,__LINE__)
     @fig.parm_set('DCL::gllset("LMISS", true )', __FILE__,__LINE__)
   end
 
@@ -378,14 +386,14 @@ class Draw
     i=0
     @gphys.lost_axes.each{|info|
       @fig.figure_set("i=#{i}", __FILE__,__LINE__)
-      @fig.figure_set("DCL::sgtxzv(vxmax+0.01,vymax-0.03*i,\"#{info}\",0.02,0,-1,1)", __FILE__,__LINE__)
+      @fig.figure_set("DCL::sgtxzr(vxmax+0.01,vymax-0.03*i,\"#{info}\",0.02,0,-1,1)", __FILE__,__LINE__)
       i+=1
     }
   end
 
   private
   def draw_1d(title)
-    @fig.figure_set('DCL::sgplzu(gphys.coord(0).val, gphys.val, 1, 3 )', __FILE__,__LINE__)
+    @fig.figure_set('DCL::sgplzu(gphys0.coord(0).val, gphys0.val, 1, 3 )', __FILE__,__LINE__)
     axis
     @fig.figure_set("title = \"#{title}\"", __FILE__,__LINE__)
     @fig.figure_set('DCL::uxsttl("t", title, 0 )', __FILE__,__LINE__)
@@ -411,42 +419,56 @@ class Draw
       x = 0
       y = 1
     end
-    @fig.figure_set("DCL::uwsgxa(gphys.coord(#{x}).val)", __FILE__,__LINE__)
-    @fig.figure_set("DCL::uwsgya(gphys.coord(#{y}).val)", __FILE__,__LINE__)
+    @fig.figure_set("DCL::uwsgxa(gphys0.coord(#{x}).val)", __FILE__,__LINE__)
+    @fig.figure_set("DCL::uwsgya(gphys0.coord(#{y}).val)", __FILE__,__LINE__)
     @fig.figure_set('DCL::ueitlv', __FILE__,__LINE__)
-    if @tone.nil?
-      min = @gphys.val.min
-      max = @gphys.val.max
-      dm = 0
-    else
-      min,dm = @tone
-      max = min+dm*(50-1)
+
+    if type == CONTOUR || type == MAP
+      if @tone.nil?
+        min = @gphys.val.min
+        max = @gphys.val.max
+        dm = 0
+      else
+        min,dm = @tone
+        max = min+dm*(50-1)
+      end
+      if min!=max then
+        @fig.figure_set("DCL::uegtla(#{min}, #{max}, #{dm} )", __FILE__,__LINE__)
+      end
+      if @transpose
+        @fig.figure_set("DCL::uetone(gphys0.val.transpose(#{x},#{y}))", __FILE__,__LINE__)
+      else
+        @fig.figure_set("DCL::uetone(gphys0.val)", __FILE__,__LINE__)
+      end
+      if @cont.nil?
+        min = @gphys.val.min
+        max = @gphys.val.max
+        dm = 0
+      else
+        min,dm = @cont
+        max = min+dm*(50-1)
+      end
+      if min!=max then
+        @fig.figure_set("DCL::udgcla(#{min}, #{max}, #{dm} )", __FILE__,__LINE__)
+      end
+      if @transpose
+        @fig.figure_set("DCL::udcntz(gphys0.val.transpose(#{x},#{y}))", __FILE__,__LINE__)
+      else
+        @fig.figure_set("DCL::udcntz(gphys0.val)", __FILE__,__LINE__)
+      end
+    elsif type == VECTOR
+      if @vector
+        @fig.figure_set("DCL::uglset('lnrmal', false)", __FILE__,__LINE__)
+        @fig.figure_set("DCL::ugrset('xfact1', #{@vector[0]})", __FILE__,__LINE__)
+        @fig.figure_set("DCL::ugrset('yfact1', #{@vector[1]})", __FILE__,__LINE__)
+      end
+      if @transpose
+        @fig.figure_set("DCL::ugvect(gphys0.val.transpose(#{x},#{y}), gphys1.val.transpose(#{x},#{y}))", __FILE__,__LINE__)
+      else
+        @fig.figure_set("DCL::ugvect(gphys0.val, gphys1.val)", __FILE__,__LINE__)
+      end
     end
-    if min!=max then
-      @fig.figure_set("DCL::uegtla(#{min}, #{max}, #{dm} )", __FILE__,__LINE__)
-    end
-    if @transpose
-      @fig.figure_set("DCL::uetonf(gphys.val.transpose(#{x},#{y}))", __FILE__,__LINE__)
-    else
-      @fig.figure_set("DCL::uetonf(gphys.val)", __FILE__,__LINE__)
-    end
-    if @cont.nil?
-      min = @gphys.val.min
-      max = @gphys.val.max
-      dm = 0
-    else
-      min,dm = @cont
-      max = min+dm*(50-1)
-    end
-    if min!=max then
-      @fig.figure_set("DCL::udgcla(#{min}, #{max}, #{dm} )", __FILE__,__LINE__)
-    end
-    if @transpose
-      @fig.figure_set("DCL::udcntz(gphys.val.transpose(#{x},#{y}))", __FILE__,__LINE__)
-    else
-      @fig.figure_set("DCL::udcntz(gphys.val)", __FILE__,__LINE__)
-    end
-    if type==CONTOUR then
+    if type==CONTOUR || type==VECTOR then
       axis
       @fig.figure_set('DCL::uzrset("ROFFXT", 0.06)', __FILE__,__LINE__)
       @fig.figure_set("title = \"#{title}\"", __FILE__,__LINE__)
@@ -486,19 +508,29 @@ class Draw
     @fig.figure_set('DCL::sglset("LCLIP", true )', __FILE__,__LINE__)
     if @type == LINE then
       draw_1d(@title)
-    elsif (@type==CONTOUR) || (@type==MAP) then
+    elsif @type==CONTOUR || @type==MAP || @type==VECTOR then
       draw_2d(@title,@type)
+      # draw map if @type==MAP
     end
-    gphys = @gphys
-#    eval(@fig.draw)
+    DCL::zgsdrw(drawing_area)
+    DCL::zgspmp(@pixmap)
+    reset
+    gphys0 = @gphys
+    gphys1 = @gphys2 if @type==VECTOR
     codes = @fig.draw_eval
-    for i in 0...codes.length
-      eval *codes[i]
+    for j in 0...codes.length
+      eval *codes[j]
     end
     expose_event(drawing_area)
   end
 
   private
+  def reset
+    wsxmn,wsxmx,wsymn,wsymx,fact = DCL::swqrct
+    DCL::stswrc(wsxmn,wsxmx,wsymn,wsymx)
+    DCL::slinit(wsxmx-wsxmn,wsymx-wsymn,fact)
+  end
+    
   def expose_event(widget,event=nil)
     if ! @pixmap.nil?
       gc = widget.style.fg_gc(widget.state)
@@ -529,9 +561,7 @@ class Draw
       DCL::sgopn(4)
       @opened = true
     else
-      wsxmn,wsxmx,wsymn,wsymx,fact = DCL::swqrct
-      DCL::stswrc(wsxmn,wsxmx,wsymn,wsymx)
-      DCL::slinit(wsxmx-wsxmn,wsymx-wsymn,fact)
+      reset
     end
     draw(widget)
     true
@@ -580,31 +610,48 @@ END
       @title[3] = 'y_unit = ""'
       @close = "DCL::grcls"
     end
-    def var_set(filename,filetype,funcs,mapping)
-      varname = funcs[0]
+    def var_set(funcs,mapping,n)
+      @var = "" if n==0
+      filename, filetype, varname = funcs[0]
       if filename.length==1
-        @var = "filename = \"#{filename[0]}\"\n"
+        @var += "filename = \"#{filename[0]}\"\n"
       else
-        @var = 'filename = ["#{filename.join('", "')}"]\n'
+        @var += "filename = [\"#{filename.join('", "')}\"]\n"
       end
       @var += "varname = \"#{varname}\"\n"
 
 
       if filetype=="netcdf"
-        @var += "gphys = GPhys::NetCDF_IO.open(filename,varname)\n"
+        @var += "gphys#{n} = GPhys::NetCDF_IO.open(filename,varname)\n"
       elsif filetype=="grads"
-        @var += "gphys = GPhys::GrADS_IO.open(filename,varname)\n"
+        @var += "gphys#{n} = GPhys::GrADS_IO.open(filename,varname)\n"
+      elsif filetype=="druby"
+        @var += "class NArray\n"
+        @var += "  def self._load(o) to_na(*Marshal::load(o)).ntoh end\n"
+        @var += "end\n"
+        @var += "ary = filename.split('/')\n"
+        @var += "host = 'druby://'+ary[2]\n"
+        @var += "path = ary[3..-1].join('/')\n"
+        @var += "tree = DRbObject.new(nil,host)\n"
+        @var += "gphys#{n} = tree.data(path)\n"
       else
-        raise "filetype is not netcdf and grads"
+        raise "filetype isn't invalid"
       end
 
       if funcs.length!=0
 	funcs[1..-1].each{|func|
-	  @var += "gphys = gphys.cut(#{func[2].join(",")})\n"
-	  @var += "gphys = gphys.#{func[0]}(#{func[1]})\n"
+	  @var += "gphys#{n} = gphys#{n}.cut(#{func[2].join(",")})\n"
+	  @var += "gphys#{n} = gphys#{n}.#{func[0]}(#{func[1]})\n"
 	}
       end
-      @var += "gphys = gphys[#{mapping.join(",")}]"
+      str = mapping.collect{|a|
+        if NArray===a
+          "NArray[#{a.to_a.join(",")}]"
+        else
+          a
+        end
+        }.join(",")
+      @var += "gphys#{n} = gphys#{n}[#{str}]\n"
     end
     def window_set(xmin,xmax,ymin,ymax, file,line)
       @window[0] = "xmin = #{xmin}"
